" Vim plugin to add auto-detection of the HTML encoding for installed Windows
" codepages (those that are well-supported) and auto-detection of any installed
" codepage from an HTML encoding.
"
" Maintainer: Ben Fritz <fritzophrenic@gmail.com>
" Version: 2

let s:all_windows_codepages = {
      \ 'c_037.nls'  :   [0, 1,
      \                   'IBM037',
      \                   'cp037',
      \                   'ebcdic-cp-us',
      \                   'ebcdic-cp-ca',
      \                   'ebcdic-cp-wt',
      \                   'ebcdic-cp-nl',
      \                   'csIBM037'],
      \ 'c_437.nls'  :   [0, 0,
      \                   'IBM437',
      \                   'cp437',
      \                   '437',
      \                   'csPC8CodePage437'],
      \ 'c_500.nls'  :   [0, 1,
      \                   'IBM500',
      \                   'CP500',
      \                   'ebcdic-cp-be',
      \                   'ebcdic-cp-ch',
      \                   'csIBM500'],
      \ 'c_708.nls'  :   [1, 0,
      \                   'ISO-8859-6',
      \                   'ISO_8859-6:1987',
      \                   'iso-ir-127',
      \                   'ISO_8859-6',
      \                   'ECMA-114',
      \                   'ASMO-708',
      \                   'arabic',
      \                   'csISOLatinArabic'],
      \ 'c_709.nls'  :   [0, 1,
      \                   'ASMO_449',
      \                   'ISO_9036',
      \                   'arabic7',
      \                   'iso-ir-89',
      \                   'csISO89ASMO449'],
      \ 'c_710.nls'  :   [0, 0],
      \ 'c_720.nls'  :   [0, 1, 'DOS-720'],
      \ 'c_737.nls'  :   [0, 1, 'ibm737'],
      \ 'c_775.nls'  :   [0, 0,
      \                   'IBM775',
      \                   'cp775',
      \                   'csPC775Baltic'],
      \ 'c_850.nls'  :   [0, 0,
      \                   'IBM850',
      \                   'cp850',
      \                   '850',
      \                   'csPC850Multilingual'],
      \ 'c_852.nls'  :   [0, 0,
      \                   'IBM852',
      \                   'cp852',
      \                   '852',
      \                   'csPCp852'],
      \ 'c_855.nls'  :   [0, 0,
      \                   'IBM855',
      \                   'cp855',
      \                   '855',
      \                   'csIBM855'],
      \ 'c_857.nls'  :   [0, 0,
      \                   'IBM857',
      \                   'cp857',
      \                   '857',
      \                   'csIBM857'],
      \ 'c_858.nls'  :   [0, 1,
      \                   'IBM00858',
      \                   'CCSID00858',
      \                   'CP00858',
      \                   'PC-Multilingual-850+euro'],
      \ 'c_860.nls'  :   [0, 0,
      \                   'IBM860',
      \                   'cp860',
      \                   '860',
      \                   'csIBM860'],
      \ 'c_861.nls'  :   [0, 0,
      \                   'IBM861',
      \                   'cp861',
      \                   '861',
      \                   'cp-is',
      \                   'csIBM861'],
      \ 'c_862.nls'  :   [0, 1, 'DOS-862'],
      \ 'c_863.nls'  :   [0, 0,
      \                   'IBM863',
      \                   'cp863',
      \                   '863',
      \                   'csIBM863'],
      \ 'c_864.nls'  :   [0, 1,
      \                   'IBM864',
      \                   'cp864',
      \                   'csIBM864'],
      \ 'c_865.nls'  :   [0, 0,
      \                   'IBM865',
      \                   'cp865',
      \                   '865',
      \                   'csIBM865'],
      \ 'c_866.nls'  :   [1, 0,
      \                   'cp866',
      \                   'IBM866',
      \                   '866',
      \                   'csIBM866'],
      \ 'c_869.nls'  :   [0, 0,
      \                   'IBM869',
      \                   'cp869',
      \                   '869',
      \                   'cp-gr',
      \                   'csIBM869'],
      \ 'c_870.nls'  :   [0, 1,
      \                   'IBM870',
      \                   'CP870',
      \                   'ebcdic-cp-roece',
      \                   'ebcdic-cp-yu',
      \                   'csIBM870'],
      \ 'c_874.nls'  :   [0, 1, 'windows-874'],
      \ 'c_875.nls'  :   [0, 1, 'cp875'],
      \ 'c_932.nls'  :   [0, 0,
      \                   'Shift_JIS',
      \                   'MS_Kanji',
      \                   'csShiftJIS'],
      \ 'c_936.nls'  :   [1, 1,
      \                   'GB2312',
      \                   'csGB2312'],
      \ 'c_949.nls'  :   [0, 0,
      \                   'KS_C_5601-1987',
      \                   'iso-ir-149',
      \                   'KS_C_5601-1989',
      \                   'KSC_5601',
      \                   'korean',
      \                   'csKSC56011987'],
      \ 'c_950.nls'  :   [0, 0,
      \                   'Big5',
      \                   'csBig5'],
      \ 'c_1026.nls' :   [0, 1,
      \                   'IBM1026',
      \                   'CP1026',
      \                   'csIBM1026'],
      \ 'c_1047.nls' :   [0, 1, 'IBM01047'],
      \ 'c_1140.nls' :   [0, 1,
      \                   'IBM01140',
      \                   'CCSID01140',
      \                   'CP01140',
      \                   'ebcdic-us-37+euro'],
      \ 'c_1141.nls' :   [0, 1,
      \                   'IBM01141',
      \                   'CCSID01141',
      \                   'CP01141',
      \                   'ebcdic-de-273+euro'],
      \ 'c_1142.nls' :   [0, 1,
      \                   'IBM01142',
      \                   'CCSID01142',
      \                   'CP01142',
      \                   'ebcdic-dk-277+euro',
      \                   'ebcdic-no-277+euro'],
      \ 'c_1143.nls' :   [0, 1,
      \                   'IBM01143',
      \                   'CCSID01143',
      \                   'CP01143',
      \                   'ebcdic-fi-278+euro',
      \                   'ebcdic-se-278+euro'],
      \ 'c_1144.nls' :   [0, 1,
      \                   'IBM01144',
      \                   'CCSID01144',
      \                   'CP01144',
      \                   'ebcdic-it-280+euro'],
      \ 'c_1145.nls' :   [0, 1,
      \                   'IBM01145',
      \                   'CCSID01145',
      \                   'CP01145',
      \                   'ebcdic-es-284+euro'],
      \ 'c_1146.nls' :   [0, 1,
      \                   'IBM01146',
      \                   'CCSID01146',
      \                   'CP01146',
      \                   'ebcdic-gb-285+euro'],
      \ 'c_1147.nls' :   [0, 1,
      \                   'IBM01147',
      \                   'CCSID01147',
      \                   'CP01147',
      \                   'ebcdic-fr-297+euro'],
      \ 'c_1148.nls' :   [0, 1,
      \                   'IBM01148',
      \                   'CCSID01148',
      \                   'CP01148',
      \                   'ebcdic-international-500+euro'],
      \ 'c_1149.nls' :   [0, 1,
      \                   'IBM01149',
      \                   'CCSID01149',
      \                   'CP01149',
      \                   'ebcdic-is-871+euro'],
      \ 'c_1200.nls' :   [1, 0, 'UTF-8', 'utf-16le'],
      \ 'c_1201.nls' :   [1, 0, 'UTF-8', 'utf-16be'],
      \ 'c_1250.nls' :   [0, 0, 'windows-1250'],
      \ 'c_1251.nls' :   [0, 0, 'windows-1251'],
      \ 'c_1252.nls' :   [1, 1, 'windows-1252'],
      \ 'c_1253.nls' :   [0, 0, 'windows-1253'],
      \ 'c_1254.nls' :   [0, 0, 'windows-1254'],
      \ 'c_1255.nls' :   [0, 0, 'windows-1255'],
      \ 'c_1256.nls' :   [0, 0, 'windows-1256'],
      \ 'c_1257.nls' :   [0, 0, 'windows-1257'],
      \ 'c_1258.nls' :   [0, 0, 'windows-1258'],
      \ 'c_1361.nls' :   [0, 1, 'Johab', 'x-johab'],
      \ 'c_10000.nls':   [1, 0,
      \                   'macintosh',
      \                   'mac',
      \                   'csMacintosh'],
      \ 'c_10001.nls':   [0, 1, 'x-mac-japanese'],
      \ 'c_10002.nls':   [0, 1, 'x-mac-chinesetrad'],
      \ 'c_10003.nls':   [0, 1, 'x-mac-korean'],
      \ 'c_10004.nls':   [0, 1, 'x-mac-arabic'],
      \ 'c_10005.nls':   [0, 1, 'x-mac-hebrew'],
      \ 'c_10006.nls':   [1, 1, 'x-mac-greek'],
      \ 'c_10007.nls':   [1, 1, 'x-mac-cyrillic'],
      \ 'c_10008.nls':   [0, 1, 'x-mac-chinesesimp'],
      \ 'c_10010.nls':   [0, 1, 'x-mac-romanian'],
      \ 'c_10017.nls':   [0, 1, 'x-mac-ukrainian'],
      \ 'c_10021.nls':   [0, 1, 'x-mac-thai'],
      \ 'c_10029.nls':   [1, 1, 'x-mac-ce'],
      \ 'c_10079.nls':   [0, 1, 'x-mac-icelandic'],
      \ 'c_10081.nls':   [1, 1, 'x-mac-turkish'],
      \ 'c_10082.nls':   [0, 1, 'x-mac-croatian'],
      \ 'c_12000.nls':   [1, 0,
      \                   'UTF-8',
      \                   'UTF-32'],
      \ 'c_12001.nls':   [1, 0,
      \                   'UTF-8',
      \                   'utf-32BE'],
      \ 'c_20000.nls':   [0, 1, 'x-Chinese_CNS'],
      \ 'c_20001.nls':   [0, 1, 'x-cp20001'],
      \ 'c_20002.nls':   [0, 1, 'x_Chinese-Eten'],
      \ 'c_20003.nls':   [0, 1, 'x-cp20003'],
      \ 'c_20004.nls':   [0, 1, 'x-cp20004'],
      \ 'c_20005.nls':   [0, 1, 'x-cp20005'],
      \ 'c_20105.nls':   [0, 1, 'x-IA5'],
      \ 'c_20106.nls':   [0, 1, 'x-IA5-German'],
      \ 'c_20107.nls':   [0, 1, 'x-IA5-Swedish'],
      \ 'c_20108.nls':   [0, 1, 'x-IA5-Norwegian'],
      \ 'c_20127.nls':   [1, 1,
      \                   'US-ASCII',
      \                   'ANSI_X3.4-1968',
      \                   'iso-ir-6',
      \                   'ANSI_X3.4-1986',
      \                   'ISO_646.irv:1991',
      \                   'ASCII',
      \                   'ISO646-US',
      \                   'us',
      \                   'IBM367',
      \                   'cp367',
      \                   'csASCII'],
      \ 'c_20261.nls':   [0, 1,
      \                   'T.61-7bit',
      \                   'x-cp20261',
      \                   'T.61',
      \                   'iso-ir-102',
      \                   'csISO102T617bit'],
      \ 'c_20269.nls':   [0, 1, 'x-cp20269'],
      \ 'c_20273.nls':   [0, 1,
      \                   'IBM273',
      \                   'CP273',
      \                   'csIBM273'],
      \ 'c_20277.nls':   [0, 1,
      \                   'IBM277',
      \                   'EBCDIC-CP-DK',
      \                   'EBCDIC-CP-NO',
      \                   'csIBM277'],
      \ 'c_20278.nls':   [0, 1,
      \                   'IBM278',
      \                   'CP278',
      \                   'ebcdic-cp-fi',
      \                   'ebcdic-cp-se',
      \                   'csIBM278'],
      \ 'c_20280.nls':   [0, 1,
      \                   'IBM280',
      \                   'CP280',
      \                   'ebcdic-cp-it',
      \                   'csIBM280'],
      \ 'c_20284.nls':   [0, 1,
      \                   'IBM284',
      \                   'CP284',
      \                   'ebcdic-cp-es',
      \                   'csIBM284'],
      \ 'c_20285.nls':   [0, 1,
      \                   'IBM285',
      \                   'CP285',
      \                   'ebcdic-cp-gb',
      \                   'csIBM285'],
      \ 'c_20290.nls':   [0, 1,
      \                   'IBM290',
      \                   'cp290',
      \                   'EBCDIC-JP-kana',
      \                   'csIBM290'],
      \ 'c_20297.nls':   [0, 1,
      \                   'IBM297',
      \                   'cp297',
      \                   'ebcdic-cp-fr',
      \                   'csIBM297'],
      \ 'c_20420.nls':   [0, 1,
      \                   'IBM420',
      \                   'cp420',
      \                   'ebcdic-cp-ar1',
      \                   'csIBM420'],
      \ 'c_20423.nls':   [0, 1,
      \                   'IBM423',
      \                   'cp423',
      \                   'ebcdic-cp-gr',
      \                   'csIBM423'],
      \ 'c_20424.nls':   [0, 1,
      \                   'IBM424',
      \                   'cp424',
      \                   'ebcdic-cp-he',
      \                   'csIBM424'],
      \ 'c_20833.nls':   [0, 1, 'x-EBCDIC-KoreanExtended'],
      \ 'c_20838.nls':   [0, 0, 'IBM-Thai'],
      \ 'c_20866.nls':   [1, 0,
      \                   'KOI8-R',
      \                   'csKOI8R'],
      \ 'c_20871.nls':   [0, 1,
      \                   'IBM871',
      \                   'CP871',
      \                   'ebcdic-cp-is',
      \                   'csIBM871'],
      \ 'c_20880.nls':   [0, 1,
      \                   'IBM880',
      \                   'cp880',
      \                   'EBCDIC-Cyrillic',
      \                   'csIBM880'],
      \ 'c_20905.nls':   [0, 1,
      \                   'IBM905',
      \                   'CP905',
      \                   'ebcdic-cp-tr',
      \                   'csIBM905'],
      \ 'c_20924.nls':   [0, 1,
      \                   'IBM00924',
      \                   'CCSID00924',
      \                   'CP00924',
      \                   'ebcdic-Latin9--euro'],
      \ 'c_20932.nls':   [0, 0,
      \                   'EUC-JP',
      \                   'Extended_UNIX_Code_Packed_Format_for_Japanese',
      \                   'csEUCPkdFmtJapanese'],
      \ 'c_20936.nls':   [1, 0,
      \                   'GB2312',
      \                   'x-cp20936',
      \                   'csGB2312'],
      \ 'c_20949.nls':   [0, 1, 'x-cp20949'],
      \ 'c_21025.nls':   [0, 1, 'cp1025'],
      \ 'c_21027.nls':   [0, 0],
      \ 'c_21866.nls':   [1, 0, 'KOI8-U'],
      \ 'c_28591.nls':   [1, 0,
      \                   'ISO-8859-1',
      \                   'ISO_8859-1:1987',
      \                   'iso-ir-100',
      \                   'ISO_8859-1',
      \                   'latin1',
      \                   'l1',
      \                   'IBM819',
      \                   'CP819',
      \                   'csISOLatin1'],
      \ 'c_28592.nls':   [1, 0,
      \                   'ISO-8859-2',
      \                   'ISO_8859-2:1987',
      \                   'iso-ir-101',
      \                   'ISO_8859-2',
      \                   'latin2',
      \                   'l2',
      \                   'csISOLatin2'],
      \ 'c_28593.nls':   [1, 0,
      \                   'ISO-8859-3',
      \                   'ISO_8859-3:1988',
      \                   'iso-ir-109',
      \                   'ISO_8859-3',
      \                   'latin3',
      \                   'l3',
      \                   'csISOLatin3'],
      \ 'c_28594.nls':   [1, 0,
      \                   'ISO-8859-4',
      \                   'ISO_8859-4:1988',
      \                   'iso-ir-110',
      \                   'ISO_8859-4',
      \                   'latin4',
      \                   'l4',
      \                   'csISOLatin4'],
      \ 'c_28595.nls':   [1, 0,
      \                   'ISO-8859-5',
      \                   'ISO_8859-5:1988',
      \                   'iso-ir-144',
      \                   'ISO_8859-5',
      \                   'cyrillic',
      \                   'csISOLatinCyrillic'],
      \ 'c_28596.nls':   [1, 0,
      \                   'ISO-8859-6',
      \                   'ISO_8859-6:1987',
      \                   'iso-ir-127',
      \                   'ISO_8859-6',
      \                   'ECMA-114',
      \                   'ASMO-708',
      \                   'arabic',
      \                   'csISOLatinArabic'],
      \ 'c_28597.nls':   [1, 0,
      \                   'ISO-8859-7',
      \                   'ISO_8859-7:1987',
      \                   'iso-ir-126',
      \                   'ISO_8859-7',
      \                   'ELOT_928',
      \                   'ECMA-118',
      \                   'greek',
      \                   'greek8',
      \                   'csISOLatinGreek'],
      \ 'c_28598.nls':   [1, 0,
      \                   'ISO-8859-8',
      \                   'ISO_8859-8:1988',
      \                   'iso-ir-138',
      \                   'ISO_8859-8',
      \                   'hebrew',
      \                   'csISOLatinHebrew'],
      \ 'c_28599.nls':   [1, 0,
      \                   'ISO-8859-9',
      \                   'ISO_8859-9:1989',
      \                   'iso-ir-148',
      \                   'ISO_8859-9',
      \                   'latin5',
      \                   'l5',
      \                   'csISOLatin5'],
      \ 'c_28603.nls':   [1, 0, 'ISO-8859-13'],
      \ 'c_28605.nls':   [1, 0,
      \                   'ISO-8859-15',
      \                   'ISO_8859-15',
      \                   'Latin-9'],
      \ 'c_29001.nls':   [0, 1, 'x-Europa'],
      \ 'c_38598.nls':   [1, 1,
      \                   'ISO-8859-8-I',
      \                   'ISO_8859-8-I',
      \                   'csISO88598I'],
      \ 'c_50220.nls':   [1, 0,
      \                   'ISO-2022-JP',
      \                   'csISO2022JP'],
      \ 'c_50221.nls':   [1, 1,
      \                   'ISO-2022-JP',
      \                   'csISO2022JP'],
      \ 'c_50222.nls':   [1, 0,
      \                   'ISO-2022-JP',
      \                   'csISO2022JP'],
      \ 'c_50225.nls':   [1, 1,
      \                   'ISO-2022-KR',
      \                   'csISO2022KR'],
      \ 'c_50227.nls':   [0, 1, 'x-cp50227'],
      \ 'c_50229.nls':   [1, 1, 'ISO-2022-CN'],
      \ 'c_50930.nls':   [0, 0],
      \ 'c_50931.nls':   [0, 0],
      \ 'c_50933.nls':   [0, 0],
      \ 'c_50935.nls':   [0, 0],
      \ 'c_50936.nls':   [0, 0],
      \ 'c_50937.nls':   [0, 0],
      \ 'c_50939.nls':   [0, 0],
      \ 'c_51932.nls':   [1, 0,
      \                   'EUC-JP',
      \                   'Extended_UNIX_Code_Packed_Format_for_Japanese',
      \                   'csEUCPkdFmtJapanese'],
      \ 'c_51936.nls':   [1, 0,
      \                   'GB_2312-80',
      \                   'iso-ir-58',
      \                   'chinese',
      \                   'csISO58GB231280'],
      \ 'c_51949.nls':   [1, 0,
      \                   'EUC-KR',
      \                   'csEUCKR'],
      \ 'c_51950.nls':   [0, 0],
      \ 'c_52936.nls':   [1, 1,
      \                   'HZ-GB-2312'],
      \ 'c_54936.nls':   [1, 1,
      \                   'GB18030'],
      \ 'c_57002.nls':   [0, 1, 'x-iscii-de'],
      \ 'c_57003.nls':   [0, 1, 'x-iscii-be'],
      \ 'c_57004.nls':   [0, 1, 'x-iscii-ta'],
      \ 'c_57005.nls':   [0, 1, 'x-iscii-te'],
      \ 'c_57006.nls':   [0, 1, 'x-iscii-as'],
      \ 'c_57007.nls':   [0, 1, 'x-iscii-or'],
      \ 'c_57008.nls':   [0, 1, 'x-iscii-ka'],
      \ 'c_57009.nls':   [0, 1, 'x-iscii-ma'],
      \ 'c_57010.nls':   [0, 1, 'x-iscii-gu'],
      \ 'c_57011.nls':   [0, 1, 'x-iscii-pa'],
      \ 'c_65000.nls':   [0, 1, 'UTF-7'],
      \ 'c_65001.nls':   [1, 0, 'UTF-8']
      \ }

function tohtml_wincp#get_installed_cps()
  if !exists('g:html_charset_override')
    let g:html_charset_override = {}
  endif
  if !exists('g:html_encoding_override')
    let g:html_encoding_override = {}
  endif

  let installed_codepages = map(split(glob($SYSTEMROOT.'\System32\c_*.nls', 1)),
        \                       'tolower(fnamemodify(v:val, ":t"))')
  for codepage in installed_codepages
    let vim_encoding=substitute(codepage, '^c_\(\d\+\)\.nls', 'cp\1', '')
    if has_key(s:all_windows_codepages, codepage)
      if s:all_windows_codepages[codepage][0] && !has_key(g:html_charset_override, vim_encoding)
        let g:html_charset_override[vim_encoding] = s:all_windows_codepages[codepage][2]
      endif
      if s:all_windows_codepages[codepage][1]
        for html_encoding in s:all_windows_codepages[codepage][2:]
          if !has_key(g:html_encoding_override, html_encoding)
            let g:html_encoding_override[html_encoding] = vim_encoding
          endif
        endfor
      endif
    endif
  endfor
  unlet s:all_windows_codepages
endfunction
